Attribute VB_Name = "Calibration"
Option Explicit
Public Function Rtd(ByVal Temp As Single, RTD_Base As Single) As Double
    'Const A = 0.0039083
    'Const B = -0.0000005775
    'Const C = -4.1835E-12
    Const RTD_LIMIT_LOW = -200
    Const RTD_LIMIT_HIGH = 850
    If Temp < RTD_LIMIT_LOW Then
        MsgBox "Minimum RTD Temperature = -200C."
        Temp = RTD_LIMIT_LOW
    ElseIf Temp > RTD_LIMIT_HIGH Then
       MsgBox "Maximum RTD Temperature = 850C."
       Temp = RTD_LIMIT_HIGH
    End If
    If Temp < 0 Then
        Rtd = RTD_Base * (1 + AK * Temp + BK * Temp ^ 2 + CK * (Temp - 100) * Temp ^ 3)
    Else
        Rtd = RTD_Base * (1 + AK * Temp + BK * Temp ^ 2)
    End If
End Function
Public Function FindTemperature(RTD_val, rtd_base2)
Dim RTD_Error, Temp, Temp_low, Temp_high, Temp_Midpoint, Rtd
RTD_Error = 1000
If ((RTD_val / rtd_base2 < 0.185) Or (RTD_val / rtd_base2 > 3.905)) Then
    MsgBox "RTD Value must be in the range: " + Str(rtd_base2 * 0.185) + "< RTD <" + Str(rtd_base2 * 3.905)
    Exit Function
End If

Temp_low = -300
Temp_high = 1000
While Abs(100 * RTD_Error / rtd_base2) > 0.001
    Temp_Midpoint = (Temp_high + Temp_low) / 2
    If Temp_Midpoint < 0 Then
        Rtd = rtd_base2 * (1 + AK * Temp_Midpoint + BK * Temp_Midpoint ^ 2 + CK * (Temp_Midpoint - 100) * Temp_Midpoint ^ 3)
    Else
        Rtd = rtd_base2 * (1 + AK * Temp_Midpoint + BK * Temp_Midpoint ^ 2)
    End If
    If Rtd < 0 Then Rtd = 0
    RTD_Error = RTD_val - Rtd
    If RTD_Error >= 0 Then
         Temp_low = Temp_Midpoint
    Else
         Temp_high = Temp_Midpoint
    End If
 
Wend
FindTemperature = Temp_Midpoint
End Function
Public Function FindBV(Rmin As Single, Rmax As Single, Rmid As Single) As Single
    FindBV = (Rmid - (Rmax + Rmin) / 2) / (Rmax - Rmin)
End Function

Public Function FindGLin(bv As Single, Rmax As Single, Rmin As Single, RZ As Single) As Single
    FindGLin = (2 * bv) / ((0.5 + bv) * Rmax - (0.5 - bv) * Rmin - 2 * bv * RZ)
End Function
Public Function FindClosestRZ(RZ() As Single, Rmin As Single) As Integer
    Dim i As Integer, j As Integer
    j = 1
    For i = 1 To 5
        If Abs(RZ(i) - Rmin) < Abs(RZ(j) - Rmin) Then
            j = i
        End If
    Next i
    FindClosestRZ = j
    'check for exception
    Call ReadFindResistorScrn

    For i = 1 To 5
    If i <> (FindClosestRZ) Then
        If RZ(i) = RZ(FindClosestRZ) Then
            If (FindRes_a(i - 1) <> FindRes_a(FindClosestRZ - 1)) Or (FindRes_b(i - 1) <> FindRes_b(FindClosestRZ - 1)) Or (FindRes_c(i - 1) <> FindRes_c(FindClosestRZ - 1)) Then
                MsgBox "You need to choose a fixed RTD range."
            End If
        End If
    End If
    Next i
End Function
Public Sub pause(PauseTime)
Dim start, roll_over
start = Timer   ' Set start time.
roll_over = Timer   ' Set start time.
    Do While (Timer < start + PauseTime) And (roll_over >= start)
        DoEvents    ' Yield to other processes.
        roll_over = Timer
    Loop
    

End Sub

'************************************************************************
'       This function calculates the appropriate current gain.
'       It returns the index of the gain, which will be put into the register
'       for the Iout DAC, Registers
Public Function CalculatePGAGain(Vmax As Single, _
                                Vmin As Single, _
                                Glin As Single, _
                                Rmax As Single, _
                                Rmin As Single, _
                                RZ As Single, _
                                Iset As Single) As Integer
Dim i As Integer
Dim Iexc As Single


For i = 0 To 6
    Iexc = calculateIexc(Vmax, Vmin, Glin, Rmax, Rmin, RZ, A(i))
    If (Abs(Iexc - (5 * Iset)) < 1.75 * Iset) Then
        Exit For
    End If
Next i
CalculatePGAGain = i
End Function
'**********************************************************
'       This calculates the desired reference currents
'***********************************************************
Public Function calculateIexc(Vmax As Single, _
                                Vmin As Single, _
                                Glin As Single, _
                                Rmax As Single, _
                                Rmin As Single, _
                                RZ As Single, _
                                A As Single) As Single
   ' old aak calculateIexc = (Vmax - Vmin) * (1 - Glin * (Rmax - RZ)) / _
                    (A * (Rmax - Rmin))
    
     calculateIexc = (Vmax - Vmin) * (1 - Glin * (Rmax - RZ)) / _
                    (A * (Rmax - Rmin))
  
                                
End Function

'*************************************************************
'           Iref (reference Current)
'               Coarse =Registers(11)
'               Fine =  Registers(10)
'               Iset =  Vref/Rset
'*************************************************************
Public Function Iref(Coarse As Integer, Fine As Integer, Iset As Single) As Single
    Iref = Iset * (5 + (Coarse / 64) + (Fine / 1024))
End Function
'************************************************************
'           HighIrefDAC
'************************************************************
Private Function HighIrefDAC(Iref As Single, Iset As Single) As Integer
    HighIrefDAC = (64 * (Iref - 5 * Iset)) / Iset
    If HighIrefDAC < -128 Then HighIrefDAC = -128   'Account for overlap.
    If HighIrefDAC > 127 Then HighIrefDAC = 127
End Function
'************************************************************
'           LowIrefDAC
'************************************************************
Private Function LowIrefDAC(Iref As Single, Coarse As Integer, Iset As Single) As Integer
    LowIrefDAC = (1024 * (Iref - 5 * Iset - (Coarse * Iset / 64)) / Iset)
End Function
'************************************************************
'           CalculateRefDACs(Iref as single, Iset as single)
'               if Registers(11) is out of range, something wrong
'*****************************************************************
Public Sub CalculateRefDACs(Iref As Single, Iset As Single)
    Registers(11) = HighIrefDAC(Iref, Iset)
    Registers(10) = LowIrefDAC(Iref, Registers(11), Iset)
End Sub


'*************************************************************
'           IZero    (Offset Current)
'                   Coarse is Registers(13)
'                   Fine is Registers(12)
'                   IzPgm is 5Vref/8RVI
'
'******************************************************************
Public Function Izero(Coarse As Integer, Fine As Integer, IzPgm As Single) As Single
        Izero = IzPgm * (35 + (Coarse / 4) + (Fine / 64))
End Function

'*****************************************************************
'           HighIzeroDac(Izero as single, Iref as single) as single
'                   IzPgm = 5Vref/8Rvi
'********************************************************************
Public Function HighIzeroDAC(Izero As Single, IzPgm As Single) As Integer
    HighIzeroDAC = (Izero - 35 * IzPgm) * 4 / IzPgm
    If HighIzeroDAC < -128 Then HighIzeroDAC = -128
    If HighIzeroDAC > 127 Then HighIzeroDAC = 127
End Function
'*****************************************************************
'           LowIzeroDac(Izero as single, Iref as single) as single
'                   Iref = 5Vref/8Rvi
'********************************************************************
Public Function LowIzeroDAC(Izero As Single, IzPgm As Single, HighDac As Integer) As Integer
    LowIzeroDAC = (Izero - 35 * IzPgm - (IzPgm * HighDac / 4)) * 64 / IzPgm
End Function
'******************************************************************
'
'           Atotal  (Gain to the output)
'               Atotal = A(i)*50/Rvi
'                        (V/V)*(A/A)/ohm
'                           A/V
'*****************************************************************
Public Function Atotal(Ai As Single, Rvi As Long) As Long
    Atotal = 50 * Ai / Rvi
End Function
'*********************************************************************
'             IArefAdj(IMax,IMin,Rvi,A,Rmax,Rmin
'                   Imax, Imin is Measured Loop Current at Tmax, Tmin'
'             CORRECTION
'
'********************************************************************
Public Function IArefAdj(IMax As Single, Imin As Single, Rvi As Long, _
                            A As Single, Rmax As Single, Rmin As Single) As Single
    
 If frmRegDetail.cbxCalDesLoop = "Current" Then
    IArefAdj = (IMax - Imin) * Rvi / (50 * A * (Rmax - Rmin))
 Else 'voltage
    IArefAdj = (IMax - Imin) * 1000 / (A * (Rmax - Rmin))
 End If
    
End Function
'*********************************************************************
'             RzAdj(Izero,IMin,Rvi,A,IrefAdj)
'                   Izero is the desired loop current at Tmin
'                   Imin is the measured loop current at Tmin'
'             CORRECTION
'********************************************************************
Public Function RzAdj(Izero As Single, Imin As Single, Rvi As Long, _
                            A As Single, IrefAdj As Single) As Single
 If frmRegDetail.cbxCalDesLoop = "Current" Then
    RzAdj = ((Izero - Imin) * Rvi) / (50 * A * IrefAdj)
 Else 'voltage
    RzAdj = ((Izero - (50 * Imin / Rvi) * 1000) * Rvi) / (50 * A * IrefAdj) 'fix Jun 13 5:02pm
 
 End If
   
End Function

'*********************************************************************
'             IBrefAdj(IMax,IMin,Rvi,A,Rmax,Rmin
'                   Imax, Imin is Measured Loop Current at Tmax, Tmin'
'             CORRECTION
'
'********************************************************************
Public Function IBrefAdj(IMax As Single, Imin As Single, Glin As Single, _
                    Rmax As Single, Rmin As Single, RzA As Single, Rvi As Long, _
                    A As Single) As Single
 If frmRegDetail.cbxCalDesLoop = "Current" Then
    IBrefAdj = ((IMax - Imin) * (1 - Glin * (Rmax - RzA)) * Rvi) / (50 * A * (Rmax - Rmin))
 Else 'voltage
    IBrefAdj = ((IMax - Imin) * (1 - Glin * (Rmax - RzA)) * 1000) / (A * (Rmax - Rmin))
 End If
End Function

